# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import (
    sera_data_navigation_get_params,
    sera_data_navigation_list_params,
    sera_data_navigation_count_params,
    sera_data_navigation_tuple_params,
    sera_data_navigation_create_params,
    sera_data_navigation_update_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncOffsetPage, AsyncOffsetPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.sera_data_navigation_get_response import SeraDataNavigationGetResponse
from ..types.sera_data_navigation_list_response import SeraDataNavigationListResponse
from ..types.sera_data_navigation_tuple_response import SeraDataNavigationTupleResponse

__all__ = ["SeraDataNavigationResource", "AsyncSeraDataNavigationResource"]


class SeraDataNavigationResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SeraDataNavigationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Bluestaq/udl-python-sdk#accessing-raw-response-data-eg-headers
        """
        return SeraDataNavigationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SeraDataNavigationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Bluestaq/udl-python-sdk#with_streaming_response
        """
        return SeraDataNavigationResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        classification_marking: str,
        data_mode: Literal["REAL", "TEST", "SIMULATED", "EXERCISE"],
        source: str,
        spacecraft_id: str,
        id: str | NotGiven = NOT_GIVEN,
        area_coverage: str | NotGiven = NOT_GIVEN,
        clock_type: str | NotGiven = NOT_GIVEN,
        hosted_for_company_org_id: str | NotGiven = NOT_GIVEN,
        id_navigation: str | NotGiven = NOT_GIVEN,
        location_accuracy: float | NotGiven = NOT_GIVEN,
        manufacturer_org_id: str | NotGiven = NOT_GIVEN,
        mode_frequency: str | NotGiven = NOT_GIVEN,
        modes: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        notes: str | NotGiven = NOT_GIVEN,
        origin: str | NotGiven = NOT_GIVEN,
        partner_spacecraft_id: str | NotGiven = NOT_GIVEN,
        payload_type: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Service operation to take a single SeradataNavigation as a POST body and ingest
        into the database. A specific role is required to perform this service
        operation. Please contact the UDL team for assistance.

        Args:
          classification_marking: Classification marking of the data in IC/CAPCO Portion-marked format.

          data_mode:
              Indicator of whether the data is EXERCISE, REAL, SIMULATED, or TEST data:

              EXERCISE:&nbsp;Data pertaining to a government or military exercise. The data
              may include both real and simulated data.

              REAL:&nbsp;Data collected or produced that pertains to real-world objects,
              events, and analysis.

              SIMULATED:&nbsp;Synthetic data generated by a model to mimic real-world
              datasets.

              TEST:&nbsp;Specific datasets used to evaluate compliance with specifications and
              requirements, and for validating technical, functional, and performance
              characteristics.

          source: Source of the data.

          spacecraft_id: Seradata ID of the spacecraft (SeradataSpacecraftDetails ID).

          id: Unique identifier of the record, auto-generated by the system.

          area_coverage: Area of coverage, e.g. Worldwide, India, etc.

          clock_type: Type of clock, e.g. Rubidium, Hydrogen Maser, etc.

          hosted_for_company_org_id: Hosted for company/Organization Id.

          id_navigation: UUID of the parent Navigation record.

          location_accuracy: Location accuracy in meters.

          manufacturer_org_id: Manufacturer Organization Id.

          mode_frequency: Frequency for this payload.

          modes: Modes of operation.

          name: Sensor name from Seradata, e.g. WAAS GEO-5, etc.

          notes: Payload notes.

          origin: Originating system or organization which produced the data, if different from
              the source. The origin may be different than the source if the source was a
              mediating system which forwarded the data on behalf of the origin system. If
              null, the source may be assumed to be the origin.

          partner_spacecraft_id: Seradata ID of the spacecraft (SeradataSpacecraftDetails ID).

          payload_type: Navigation payload type, e.g. WAAS, GAGAN, etc.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/udl/seradatanavigation",
            body=maybe_transform(
                {
                    "classification_marking": classification_marking,
                    "data_mode": data_mode,
                    "source": source,
                    "spacecraft_id": spacecraft_id,
                    "id": id,
                    "area_coverage": area_coverage,
                    "clock_type": clock_type,
                    "hosted_for_company_org_id": hosted_for_company_org_id,
                    "id_navigation": id_navigation,
                    "location_accuracy": location_accuracy,
                    "manufacturer_org_id": manufacturer_org_id,
                    "mode_frequency": mode_frequency,
                    "modes": modes,
                    "name": name,
                    "notes": notes,
                    "origin": origin,
                    "partner_spacecraft_id": partner_spacecraft_id,
                    "payload_type": payload_type,
                },
                sera_data_navigation_create_params.SeraDataNavigationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def update(
        self,
        path_id: str,
        *,
        classification_marking: str,
        data_mode: Literal["REAL", "TEST", "SIMULATED", "EXERCISE"],
        source: str,
        spacecraft_id: str,
        body_id: str | NotGiven = NOT_GIVEN,
        area_coverage: str | NotGiven = NOT_GIVEN,
        clock_type: str | NotGiven = NOT_GIVEN,
        hosted_for_company_org_id: str | NotGiven = NOT_GIVEN,
        id_navigation: str | NotGiven = NOT_GIVEN,
        location_accuracy: float | NotGiven = NOT_GIVEN,
        manufacturer_org_id: str | NotGiven = NOT_GIVEN,
        mode_frequency: str | NotGiven = NOT_GIVEN,
        modes: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        notes: str | NotGiven = NOT_GIVEN,
        origin: str | NotGiven = NOT_GIVEN,
        partner_spacecraft_id: str | NotGiven = NOT_GIVEN,
        payload_type: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """Service operation to update an SeradataNavigation.

        A specific role is required
        to perform this service operation. Please contact the UDL team for assistance.

        Args:
          classification_marking: Classification marking of the data in IC/CAPCO Portion-marked format.

          data_mode:
              Indicator of whether the data is EXERCISE, REAL, SIMULATED, or TEST data:

              EXERCISE:&nbsp;Data pertaining to a government or military exercise. The data
              may include both real and simulated data.

              REAL:&nbsp;Data collected or produced that pertains to real-world objects,
              events, and analysis.

              SIMULATED:&nbsp;Synthetic data generated by a model to mimic real-world
              datasets.

              TEST:&nbsp;Specific datasets used to evaluate compliance with specifications and
              requirements, and for validating technical, functional, and performance
              characteristics.

          source: Source of the data.

          spacecraft_id: Seradata ID of the spacecraft (SeradataSpacecraftDetails ID).

          body_id: Unique identifier of the record, auto-generated by the system.

          area_coverage: Area of coverage, e.g. Worldwide, India, etc.

          clock_type: Type of clock, e.g. Rubidium, Hydrogen Maser, etc.

          hosted_for_company_org_id: Hosted for company/Organization Id.

          id_navigation: UUID of the parent Navigation record.

          location_accuracy: Location accuracy in meters.

          manufacturer_org_id: Manufacturer Organization Id.

          mode_frequency: Frequency for this payload.

          modes: Modes of operation.

          name: Sensor name from Seradata, e.g. WAAS GEO-5, etc.

          notes: Payload notes.

          origin: Originating system or organization which produced the data, if different from
              the source. The origin may be different than the source if the source was a
              mediating system which forwarded the data on behalf of the origin system. If
              null, the source may be assumed to be the origin.

          partner_spacecraft_id: Seradata ID of the spacecraft (SeradataSpacecraftDetails ID).

          payload_type: Navigation payload type, e.g. WAAS, GAGAN, etc.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not path_id:
            raise ValueError(f"Expected a non-empty value for `path_id` but received {path_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            f"/udl/seradatanavigation/{path_id}",
            body=maybe_transform(
                {
                    "classification_marking": classification_marking,
                    "data_mode": data_mode,
                    "source": source,
                    "spacecraft_id": spacecraft_id,
                    "body_id": body_id,
                    "area_coverage": area_coverage,
                    "clock_type": clock_type,
                    "hosted_for_company_org_id": hosted_for_company_org_id,
                    "id_navigation": id_navigation,
                    "location_accuracy": location_accuracy,
                    "manufacturer_org_id": manufacturer_org_id,
                    "mode_frequency": mode_frequency,
                    "modes": modes,
                    "name": name,
                    "notes": notes,
                    "origin": origin,
                    "partner_spacecraft_id": partner_spacecraft_id,
                    "payload_type": payload_type,
                },
                sera_data_navigation_update_params.SeraDataNavigationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list(
        self,
        *,
        first_result: int | NotGiven = NOT_GIVEN,
        max_results: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncOffsetPage[SeraDataNavigationListResponse]:
        """
        Service operation to dynamically query data by a variety of query parameters not
        specified in this API documentation. See the queryhelp operation
        (/udl/&lt;datatype&gt;/queryhelp) for more details on valid/required query
        parameter information.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/udl/seradatanavigation",
            page=SyncOffsetPage[SeraDataNavigationListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "first_result": first_result,
                        "max_results": max_results,
                    },
                    sera_data_navigation_list_params.SeraDataNavigationListParams,
                ),
            ),
            model=SeraDataNavigationListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Service operation to delete an SeradataNavigation specified by the passed ID
        path parameter. A specific role is required to perform this service operation.
        Please contact the UDL team for assistance.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/udl/seradatanavigation/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def count(
        self,
        *,
        first_result: int | NotGiven = NOT_GIVEN,
        max_results: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> str:
        """
        Service operation to return the count of records satisfying the specified query
        parameters. This operation is useful to determine how many records pass a
        particular query criteria without retrieving large amounts of data. See the
        queryhelp operation (/udl/&lt;datatype&gt;/queryhelp) for more details on
        valid/required query parameter information.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "text/plain", **(extra_headers or {})}
        return self._get(
            "/udl/seradatanavigation/count",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "first_result": first_result,
                        "max_results": max_results,
                    },
                    sera_data_navigation_count_params.SeraDataNavigationCountParams,
                ),
            ),
            cast_to=str,
        )

    def get(
        self,
        id: str,
        *,
        first_result: int | NotGiven = NOT_GIVEN,
        max_results: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SeraDataNavigationGetResponse:
        """
        Service operation to get a single SeradataNavigation by its unique ID passed as
        a path parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/udl/seradatanavigation/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "first_result": first_result,
                        "max_results": max_results,
                    },
                    sera_data_navigation_get_params.SeraDataNavigationGetParams,
                ),
            ),
            cast_to=SeraDataNavigationGetResponse,
        )

    def queryhelp(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Service operation to provide detailed information on available dynamic query
        parameters for a particular data type.
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            "/udl/seradatanavigation/queryhelp",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def tuple(
        self,
        *,
        columns: str,
        first_result: int | NotGiven = NOT_GIVEN,
        max_results: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SeraDataNavigationTupleResponse:
        """
        Service operation to dynamically query data and only return specified
        columns/fields. Requested columns are specified by the 'columns' query parameter
        and should be a comma separated list of valid fields for the specified data
        type. classificationMarking is always returned. See the queryhelp operation
        (/udl/<datatype>/queryhelp) for more details on valid/required query parameter
        information. An example URI: /udl/elset/tuple?columns=satNo,period&epoch=>now-5
        hours would return the satNo and period of elsets with an epoch greater than 5
        hours ago.

        Args:
          columns: Comma-separated list of valid field names for this data type to be returned in
              the response. Only the fields specified will be returned as well as the
              classification marking of the data, if applicable. See the ‘queryhelp’ operation
              for a complete list of possible fields.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/udl/seradatanavigation/tuple",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "columns": columns,
                        "first_result": first_result,
                        "max_results": max_results,
                    },
                    sera_data_navigation_tuple_params.SeraDataNavigationTupleParams,
                ),
            ),
            cast_to=SeraDataNavigationTupleResponse,
        )


class AsyncSeraDataNavigationResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSeraDataNavigationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Bluestaq/udl-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncSeraDataNavigationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSeraDataNavigationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Bluestaq/udl-python-sdk#with_streaming_response
        """
        return AsyncSeraDataNavigationResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        classification_marking: str,
        data_mode: Literal["REAL", "TEST", "SIMULATED", "EXERCISE"],
        source: str,
        spacecraft_id: str,
        id: str | NotGiven = NOT_GIVEN,
        area_coverage: str | NotGiven = NOT_GIVEN,
        clock_type: str | NotGiven = NOT_GIVEN,
        hosted_for_company_org_id: str | NotGiven = NOT_GIVEN,
        id_navigation: str | NotGiven = NOT_GIVEN,
        location_accuracy: float | NotGiven = NOT_GIVEN,
        manufacturer_org_id: str | NotGiven = NOT_GIVEN,
        mode_frequency: str | NotGiven = NOT_GIVEN,
        modes: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        notes: str | NotGiven = NOT_GIVEN,
        origin: str | NotGiven = NOT_GIVEN,
        partner_spacecraft_id: str | NotGiven = NOT_GIVEN,
        payload_type: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Service operation to take a single SeradataNavigation as a POST body and ingest
        into the database. A specific role is required to perform this service
        operation. Please contact the UDL team for assistance.

        Args:
          classification_marking: Classification marking of the data in IC/CAPCO Portion-marked format.

          data_mode:
              Indicator of whether the data is EXERCISE, REAL, SIMULATED, or TEST data:

              EXERCISE:&nbsp;Data pertaining to a government or military exercise. The data
              may include both real and simulated data.

              REAL:&nbsp;Data collected or produced that pertains to real-world objects,
              events, and analysis.

              SIMULATED:&nbsp;Synthetic data generated by a model to mimic real-world
              datasets.

              TEST:&nbsp;Specific datasets used to evaluate compliance with specifications and
              requirements, and for validating technical, functional, and performance
              characteristics.

          source: Source of the data.

          spacecraft_id: Seradata ID of the spacecraft (SeradataSpacecraftDetails ID).

          id: Unique identifier of the record, auto-generated by the system.

          area_coverage: Area of coverage, e.g. Worldwide, India, etc.

          clock_type: Type of clock, e.g. Rubidium, Hydrogen Maser, etc.

          hosted_for_company_org_id: Hosted for company/Organization Id.

          id_navigation: UUID of the parent Navigation record.

          location_accuracy: Location accuracy in meters.

          manufacturer_org_id: Manufacturer Organization Id.

          mode_frequency: Frequency for this payload.

          modes: Modes of operation.

          name: Sensor name from Seradata, e.g. WAAS GEO-5, etc.

          notes: Payload notes.

          origin: Originating system or organization which produced the data, if different from
              the source. The origin may be different than the source if the source was a
              mediating system which forwarded the data on behalf of the origin system. If
              null, the source may be assumed to be the origin.

          partner_spacecraft_id: Seradata ID of the spacecraft (SeradataSpacecraftDetails ID).

          payload_type: Navigation payload type, e.g. WAAS, GAGAN, etc.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/udl/seradatanavigation",
            body=await async_maybe_transform(
                {
                    "classification_marking": classification_marking,
                    "data_mode": data_mode,
                    "source": source,
                    "spacecraft_id": spacecraft_id,
                    "id": id,
                    "area_coverage": area_coverage,
                    "clock_type": clock_type,
                    "hosted_for_company_org_id": hosted_for_company_org_id,
                    "id_navigation": id_navigation,
                    "location_accuracy": location_accuracy,
                    "manufacturer_org_id": manufacturer_org_id,
                    "mode_frequency": mode_frequency,
                    "modes": modes,
                    "name": name,
                    "notes": notes,
                    "origin": origin,
                    "partner_spacecraft_id": partner_spacecraft_id,
                    "payload_type": payload_type,
                },
                sera_data_navigation_create_params.SeraDataNavigationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def update(
        self,
        path_id: str,
        *,
        classification_marking: str,
        data_mode: Literal["REAL", "TEST", "SIMULATED", "EXERCISE"],
        source: str,
        spacecraft_id: str,
        body_id: str | NotGiven = NOT_GIVEN,
        area_coverage: str | NotGiven = NOT_GIVEN,
        clock_type: str | NotGiven = NOT_GIVEN,
        hosted_for_company_org_id: str | NotGiven = NOT_GIVEN,
        id_navigation: str | NotGiven = NOT_GIVEN,
        location_accuracy: float | NotGiven = NOT_GIVEN,
        manufacturer_org_id: str | NotGiven = NOT_GIVEN,
        mode_frequency: str | NotGiven = NOT_GIVEN,
        modes: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        notes: str | NotGiven = NOT_GIVEN,
        origin: str | NotGiven = NOT_GIVEN,
        partner_spacecraft_id: str | NotGiven = NOT_GIVEN,
        payload_type: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """Service operation to update an SeradataNavigation.

        A specific role is required
        to perform this service operation. Please contact the UDL team for assistance.

        Args:
          classification_marking: Classification marking of the data in IC/CAPCO Portion-marked format.

          data_mode:
              Indicator of whether the data is EXERCISE, REAL, SIMULATED, or TEST data:

              EXERCISE:&nbsp;Data pertaining to a government or military exercise. The data
              may include both real and simulated data.

              REAL:&nbsp;Data collected or produced that pertains to real-world objects,
              events, and analysis.

              SIMULATED:&nbsp;Synthetic data generated by a model to mimic real-world
              datasets.

              TEST:&nbsp;Specific datasets used to evaluate compliance with specifications and
              requirements, and for validating technical, functional, and performance
              characteristics.

          source: Source of the data.

          spacecraft_id: Seradata ID of the spacecraft (SeradataSpacecraftDetails ID).

          body_id: Unique identifier of the record, auto-generated by the system.

          area_coverage: Area of coverage, e.g. Worldwide, India, etc.

          clock_type: Type of clock, e.g. Rubidium, Hydrogen Maser, etc.

          hosted_for_company_org_id: Hosted for company/Organization Id.

          id_navigation: UUID of the parent Navigation record.

          location_accuracy: Location accuracy in meters.

          manufacturer_org_id: Manufacturer Organization Id.

          mode_frequency: Frequency for this payload.

          modes: Modes of operation.

          name: Sensor name from Seradata, e.g. WAAS GEO-5, etc.

          notes: Payload notes.

          origin: Originating system or organization which produced the data, if different from
              the source. The origin may be different than the source if the source was a
              mediating system which forwarded the data on behalf of the origin system. If
              null, the source may be assumed to be the origin.

          partner_spacecraft_id: Seradata ID of the spacecraft (SeradataSpacecraftDetails ID).

          payload_type: Navigation payload type, e.g. WAAS, GAGAN, etc.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not path_id:
            raise ValueError(f"Expected a non-empty value for `path_id` but received {path_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            f"/udl/seradatanavigation/{path_id}",
            body=await async_maybe_transform(
                {
                    "classification_marking": classification_marking,
                    "data_mode": data_mode,
                    "source": source,
                    "spacecraft_id": spacecraft_id,
                    "body_id": body_id,
                    "area_coverage": area_coverage,
                    "clock_type": clock_type,
                    "hosted_for_company_org_id": hosted_for_company_org_id,
                    "id_navigation": id_navigation,
                    "location_accuracy": location_accuracy,
                    "manufacturer_org_id": manufacturer_org_id,
                    "mode_frequency": mode_frequency,
                    "modes": modes,
                    "name": name,
                    "notes": notes,
                    "origin": origin,
                    "partner_spacecraft_id": partner_spacecraft_id,
                    "payload_type": payload_type,
                },
                sera_data_navigation_update_params.SeraDataNavigationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list(
        self,
        *,
        first_result: int | NotGiven = NOT_GIVEN,
        max_results: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[SeraDataNavigationListResponse, AsyncOffsetPage[SeraDataNavigationListResponse]]:
        """
        Service operation to dynamically query data by a variety of query parameters not
        specified in this API documentation. See the queryhelp operation
        (/udl/&lt;datatype&gt;/queryhelp) for more details on valid/required query
        parameter information.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/udl/seradatanavigation",
            page=AsyncOffsetPage[SeraDataNavigationListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "first_result": first_result,
                        "max_results": max_results,
                    },
                    sera_data_navigation_list_params.SeraDataNavigationListParams,
                ),
            ),
            model=SeraDataNavigationListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Service operation to delete an SeradataNavigation specified by the passed ID
        path parameter. A specific role is required to perform this service operation.
        Please contact the UDL team for assistance.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/udl/seradatanavigation/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def count(
        self,
        *,
        first_result: int | NotGiven = NOT_GIVEN,
        max_results: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> str:
        """
        Service operation to return the count of records satisfying the specified query
        parameters. This operation is useful to determine how many records pass a
        particular query criteria without retrieving large amounts of data. See the
        queryhelp operation (/udl/&lt;datatype&gt;/queryhelp) for more details on
        valid/required query parameter information.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "text/plain", **(extra_headers or {})}
        return await self._get(
            "/udl/seradatanavigation/count",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "first_result": first_result,
                        "max_results": max_results,
                    },
                    sera_data_navigation_count_params.SeraDataNavigationCountParams,
                ),
            ),
            cast_to=str,
        )

    async def get(
        self,
        id: str,
        *,
        first_result: int | NotGiven = NOT_GIVEN,
        max_results: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SeraDataNavigationGetResponse:
        """
        Service operation to get a single SeradataNavigation by its unique ID passed as
        a path parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/udl/seradatanavigation/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "first_result": first_result,
                        "max_results": max_results,
                    },
                    sera_data_navigation_get_params.SeraDataNavigationGetParams,
                ),
            ),
            cast_to=SeraDataNavigationGetResponse,
        )

    async def queryhelp(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Service operation to provide detailed information on available dynamic query
        parameters for a particular data type.
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            "/udl/seradatanavigation/queryhelp",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def tuple(
        self,
        *,
        columns: str,
        first_result: int | NotGiven = NOT_GIVEN,
        max_results: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SeraDataNavigationTupleResponse:
        """
        Service operation to dynamically query data and only return specified
        columns/fields. Requested columns are specified by the 'columns' query parameter
        and should be a comma separated list of valid fields for the specified data
        type. classificationMarking is always returned. See the queryhelp operation
        (/udl/<datatype>/queryhelp) for more details on valid/required query parameter
        information. An example URI: /udl/elset/tuple?columns=satNo,period&epoch=>now-5
        hours would return the satNo and period of elsets with an epoch greater than 5
        hours ago.

        Args:
          columns: Comma-separated list of valid field names for this data type to be returned in
              the response. Only the fields specified will be returned as well as the
              classification marking of the data, if applicable. See the ‘queryhelp’ operation
              for a complete list of possible fields.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/udl/seradatanavigation/tuple",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "columns": columns,
                        "first_result": first_result,
                        "max_results": max_results,
                    },
                    sera_data_navigation_tuple_params.SeraDataNavigationTupleParams,
                ),
            ),
            cast_to=SeraDataNavigationTupleResponse,
        )


class SeraDataNavigationResourceWithRawResponse:
    def __init__(self, sera_data_navigation: SeraDataNavigationResource) -> None:
        self._sera_data_navigation = sera_data_navigation

        self.create = to_raw_response_wrapper(
            sera_data_navigation.create,
        )
        self.update = to_raw_response_wrapper(
            sera_data_navigation.update,
        )
        self.list = to_raw_response_wrapper(
            sera_data_navigation.list,
        )
        self.delete = to_raw_response_wrapper(
            sera_data_navigation.delete,
        )
        self.count = to_raw_response_wrapper(
            sera_data_navigation.count,
        )
        self.get = to_raw_response_wrapper(
            sera_data_navigation.get,
        )
        self.queryhelp = to_raw_response_wrapper(
            sera_data_navigation.queryhelp,
        )
        self.tuple = to_raw_response_wrapper(
            sera_data_navigation.tuple,
        )


class AsyncSeraDataNavigationResourceWithRawResponse:
    def __init__(self, sera_data_navigation: AsyncSeraDataNavigationResource) -> None:
        self._sera_data_navigation = sera_data_navigation

        self.create = async_to_raw_response_wrapper(
            sera_data_navigation.create,
        )
        self.update = async_to_raw_response_wrapper(
            sera_data_navigation.update,
        )
        self.list = async_to_raw_response_wrapper(
            sera_data_navigation.list,
        )
        self.delete = async_to_raw_response_wrapper(
            sera_data_navigation.delete,
        )
        self.count = async_to_raw_response_wrapper(
            sera_data_navigation.count,
        )
        self.get = async_to_raw_response_wrapper(
            sera_data_navigation.get,
        )
        self.queryhelp = async_to_raw_response_wrapper(
            sera_data_navigation.queryhelp,
        )
        self.tuple = async_to_raw_response_wrapper(
            sera_data_navigation.tuple,
        )


class SeraDataNavigationResourceWithStreamingResponse:
    def __init__(self, sera_data_navigation: SeraDataNavigationResource) -> None:
        self._sera_data_navigation = sera_data_navigation

        self.create = to_streamed_response_wrapper(
            sera_data_navigation.create,
        )
        self.update = to_streamed_response_wrapper(
            sera_data_navigation.update,
        )
        self.list = to_streamed_response_wrapper(
            sera_data_navigation.list,
        )
        self.delete = to_streamed_response_wrapper(
            sera_data_navigation.delete,
        )
        self.count = to_streamed_response_wrapper(
            sera_data_navigation.count,
        )
        self.get = to_streamed_response_wrapper(
            sera_data_navigation.get,
        )
        self.queryhelp = to_streamed_response_wrapper(
            sera_data_navigation.queryhelp,
        )
        self.tuple = to_streamed_response_wrapper(
            sera_data_navigation.tuple,
        )


class AsyncSeraDataNavigationResourceWithStreamingResponse:
    def __init__(self, sera_data_navigation: AsyncSeraDataNavigationResource) -> None:
        self._sera_data_navigation = sera_data_navigation

        self.create = async_to_streamed_response_wrapper(
            sera_data_navigation.create,
        )
        self.update = async_to_streamed_response_wrapper(
            sera_data_navigation.update,
        )
        self.list = async_to_streamed_response_wrapper(
            sera_data_navigation.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            sera_data_navigation.delete,
        )
        self.count = async_to_streamed_response_wrapper(
            sera_data_navigation.count,
        )
        self.get = async_to_streamed_response_wrapper(
            sera_data_navigation.get,
        )
        self.queryhelp = async_to_streamed_response_wrapper(
            sera_data_navigation.queryhelp,
        )
        self.tuple = async_to_streamed_response_wrapper(
            sera_data_navigation.tuple,
        )
